# coding=utf-8
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2020 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#
import time
import warnings
from typing import Optional, Tuple

try:
    import MySQLdb
except ImportError:
    MySQLdb = None

from clcommon.cpapi import db_access
from clcommon.cpapi.cpapiexceptions import NoDBAccessData, NotSupported

from .collector_base import CollectorBase


def _get_mysql_queries_num(dblogin: str, dbpass: str):
    """
    Retrieve MySQL queries (number)
    :param dblogin: admin login to DB
    :param dbpass: admin password to DB
    :return: (message, queries_num) tuple.
        message - 'OK' or error message
        queries_num - active MySQL queries number or 0 if DB error
    """
    if not MySQLdb:
        return "No MySQL client libraries installed", 0
    db = None
    try:
        db = MySQLdb.connect(host='localhost', user=dblogin, passwd=dbpass, db='mysql', use_unicode=True,
                             charset='utf8')
        cursor = db.cursor()
        # This query returns current running SQL queries number.
        # See https://stackoverflow.com/questions/7432241/mysql-show-status-active-or-total-connections
        # -1 needed to hide this query from output
        with warnings.catch_warnings():
            warnings.filterwarnings('ignore', category=MySQLdb.Warning)
            cursor.execute('SELECT count(*)-1 FROM information_schema.PROCESSLIST WHERE command=\'Query\'')
        data = cursor.fetchall()
    except MySQLdb.MySQLError as e:
        return str(e), 0
    finally:
        if db is not None:
            db.close()
    return 'OK', data[0][0]


class MySQLCollector(CollectorBase):
    class DBAccess:
        def __init__(self, login: Optional[str] = None, password: Optional[str] = None):
            self.login = login
            self.password = password
            self.expire_time = time.time() + 3600

        def __eq__(self, other):
            return self.login == other.login and self.password == other.password

        def __bool__(self):
            return bool(self.login and self.password and not self.is_expired())

        def is_expired(self) -> bool:
            return time.time() > self.expire_time

    def __init__(self, _logger):
        super(MySQLCollector, self).__init__(_logger)
        self._is_mysql_error = False
        self.access = self.DBAccess()

    def init(self):
        """
        Initialize MySQL collector
        :return: None
        """
        self._aggregated_data = 0
        self._aggregated_data_count = 0
        self._logger.info("MySQL collector init")

    def _get_db_access(self) -> Tuple[str, str]:
        """
        Get DB access data from cpapi function. Logs error
        :return: tuple (db_root_login, db_password)
            None, None if error
        """
        if not self.access:
            try:
                access = db_access()    # {'pass': str, 'login': str}
                self.access = self.DBAccess(access['login'], access['pass'])
            except (NoDBAccessData, NotSupported, KeyError, TypeError) as e:
                # Can't retrieve data
                if not self._is_mysql_error:
                    self._logger.warn("MySQL collector: can't obtain MySQL DB access: %s", str(e))
                    self._is_mysql_error = True
                    self._aggregated_data_count = 0
        return self.access.login, self.access.password

    def aggregate_new_data(self):
        """
        Retrive and aggregate new data
        :return None
        """
        self._get_db_access()
        if not self.access:     # access data still is not set
            return
        # New data present - aggregate
        message, queries_num = _get_mysql_queries_num(self.access.login, self.access.password)
        if message == 'OK':
            self._aggregated_data += queries_num
            self._aggregated_data_count += 1
            self._is_mysql_error = False
            return
        # Retrieve data error
        if not self._is_mysql_error:
            self._logger.warn("MySQL collector can't obtain MySQL queries number: %s", message)
            self._is_mysql_error = True
            self._aggregated_data_count = 0

    def get_averages(self):
        """
        Get collector's averages data
        :return: dict:
            { "mysql_queries_num": 16}
            or None if can't get data
        """
        if self._aggregated_data_count == 0:
            self._aggregated_data = 0
            self._aggregated_data_count = 0
            return None
        mysql_queries_num = self._aggregated_data // self._aggregated_data_count
        self._aggregated_data = 0
        self._aggregated_data_count = 0
        return {"mysql_queries_num": mysql_queries_num}
